#include "dashboard-widget.h"

struct _DiyaDashboardWidget
{
    DiyaShellWindow parent;
    GtkWidget *revealer;
    GtkWidget *search_entry;
    bool active;
};

enum
{
    NO_PROP,
    PROP_ACTIVE,
    N_PROPERTIES
};

static GParamSpec *g_prop[N_PROPERTIES] = {0};


G_DEFINE_TYPE (DiyaDashboardWidget, diya_dashboard_widget, DIYA_TYPE_SHELL_WINDOW)

static void diya_dashboard_widget_dispose(GObject* object)
{
    (void) object;
    g_debug("diya_dashboard_widget_dispose");
    G_OBJECT_CLASS(diya_dashboard_widget_parent_class)->dispose(object);
}

static void diya_dashboard_widget_init(DiyaDashboardWidget * self)
{
    g_debug("diya_dashboard_widget_init");
    gtk_widget_init_template (GTK_WIDGET(self));
    self->active = false;
    
    // int layer shell for window
    gtk_layer_init_for_window (GTK_WINDOW(self));
    // anchor window to all edges
    gtk_layer_set_anchor (GTK_WINDOW(self), GTK_LAYER_SHELL_EDGE_LEFT, true);
    gtk_layer_set_anchor (GTK_WINDOW(self), GTK_LAYER_SHELL_EDGE_RIGHT, true);
    gtk_layer_set_anchor (GTK_WINDOW(self), GTK_LAYER_SHELL_EDGE_TOP, true);
    gtk_layer_set_anchor (GTK_WINDOW(self), GTK_LAYER_SHELL_EDGE_BOTTOM, true);
    gtk_layer_set_namespace(GTK_WINDOW(self), "diya-dashboard");
    // set margin on window
    for (int i = 0; i < GTK_LAYER_SHELL_EDGE_ENTRY_NUMBER; i++)
        gtk_layer_set_margin (GTK_WINDOW(self), i, 0);
    gtk_layer_set_layer (GTK_WINDOW(self), GTK_LAYER_SHELL_LAYER_TOP);
    //gtk_layer_set_keyboard_mode (GTK_WINDOW(self), GTK_LAYER_SHELL_KEYBOARD_MODE_ON_DEMAND);
    // the top launcher shall be exclusive
    //gtk_layer_auto_exclusive_zone_enable (GTK_WINDOW(self));
    gtk_layer_set_keyboard_mode(GTK_WINDOW(self), GTK_LAYER_SHELL_KEYBOARD_MODE_ON_DEMAND);
    // gtk_widget_set_can_focus(GTK_WIDGET(self), true);
    //gtk_widget_set_name(GTK_WIDGET(dashboard),NAMESPACE);
    //gtk_window_set_default_size(GTK_WINDOW(dashboard), 48, 48);

    // event controller
    //GtkEventController *event_controller = gtk_event_controller_key_new();
    //g_signal_connect(event_controller, "key-pressed", G_CALLBACK(on_diya_dashboard_key_press), self);
    // gtk_widget_add_controller(GTK_WIDGET(self), event_controller);
}

static void diya_dashboard_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
{
    DiyaDashboardWidget * self = DIYA_DASHBOARD_WIDGET(object);

    switch (property_id)
    {
    case PROP_ACTIVE:
    {
        gboolean active = g_value_get_boolean(value);
        if(self->active == active)
        {
            return;
        }
        self->active = active;
        if(self->active)
        {
            gtk_window_present(GTK_WINDOW(self));
            // gtk_widget_set_visible(GTK_WIDGET(self), true);
            gtk_revealer_set_reveal_child(GTK_REVEALER(self->revealer), true);
            gtk_window_set_focus(GTK_WINDOW(self), self->search_entry);
            // gtk_widget_grab_focus( GTK_WIDGET(self->search_entry));
        }
        else
        {
            gtk_revealer_set_reveal_child(GTK_REVEALER(self->revealer), false);
            gtk_widget_set_visible(GTK_WIDGET(self), false);
        }
        break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}

static void diya_dashboard_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
{
    DiyaDashboardWidget * self = DIYA_DASHBOARD_WIDGET(object);
    switch (property_id)
    {
    case PROP_ACTIVE:
        g_value_set_boolean(value, self->active);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}

static void diya_dashboard_launch(GtkButton* btn, DiyaDashboardWidget* self)
{
    g_debug("Clicked %d %d", DIYA_IS_DASHBOARD_WIDGET(self), GTK_IS_BUTTON(btn));

    GError* error = NULL;
    GAppInfo* app = g_app_info_create_from_commandline("htop", "Foot", G_APP_INFO_CREATE_NEEDS_TERMINAL, &error);
    if(error)
    {
        g_critical("Unable to create app info: %s", error->message);
        g_error_free(error);
        return;
    }
    error = NULL;
    g_app_info_launch(app, NULL, NULL, &error);
    if(error)
    {
        g_critical("Unable to launch application: %s", error->message);
        g_error_free(error);
        return;
    }
    g_object_set(self, DIYA_PROP_DASHBOARD_ACTIVE, false, NULL);
}

static void diya_dashboard_widget_class_init(DiyaDashboardWidgetClass* class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(class);
    gobject_class->dispose = diya_dashboard_widget_dispose;
    gobject_class->set_property = diya_dashboard_set_property;
    gobject_class->get_property = diya_dashboard_get_property;

    //DiyaShellWindowClass* base_class = DIYA_SHELL_WINDOW_CLASS(class);
    //base_class->setup = diya_dashboard_widget_setup;

    gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (class),"/dev/iohub/diya/shell/dashboard.ui");
    gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (class), DiyaDashboardWidget, revealer);
    gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (class), DiyaDashboardWidget, search_entry);

    gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (class), diya_dashboard_launch);

    gtk_widget_class_set_css_name(GTK_WIDGET_CLASS(class), "diya-dashboard");
    //gtk_widget_class_bind_template_child(GTK_WIDGET_CLASS (class), ExampleAppWindow, stack);
    //gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (class), search_text_changed);

    g_prop[PROP_ACTIVE] = g_param_spec_boolean(DIYA_PROP_DASHBOARD_ACTIVE, NULL, "Active the dashboard", false, G_PARAM_READWRITE); // 
 
    g_object_class_install_properties (gobject_class, N_PROPERTIES, g_prop);
}

gboolean diya_dashboard_is_active(DiyaDashboardWidget* self)
{
    return self->active;
}