#include "taskbar-widget.h"


struct _DiyaTaskbarWidget
{
    DiyaShellWindow parent;
    GtkWidget *btn_toggle;
    bool active_dashboard;
};

enum
{
    NO_PROP,
    PROP_SHOW_DASHBOARD,
    N_PROPERTIES
};

static GParamSpec *g_prop[N_PROPERTIES] = {0};


G_DEFINE_TYPE (DiyaTaskbarWidget, diya_taskbar_widget, DIYA_TYPE_SHELL_WINDOW)

static void diya_taskbar_widget_dispose(GObject* object)
{
    (void) object;
    g_debug("diya_taskbar_widget_dispose");
    G_OBJECT_CLASS(diya_taskbar_widget_parent_class)->dispose(object);
}

static void diya_taskbar_widget_init(DiyaTaskbarWidget * self)
{
    g_debug("diya_taskbar_widget_init");
    gtk_widget_init_template (GTK_WIDGET(self));
    self->active_dashboard = false;
    //builder = gtk_builder_new_from_resource ("/org/gtk/exampleapp/gears-menu.ui");
    // GtkWindow* win = GTK_WINDOW(self);

    // g_object_bind_property(self->btn_toggle, "active", self->revealer, "reveal-child", G_SETTINGS_BIND_DEFAULT);
    // gtk_widget_set_can_focus(self->revealer, false);

    //g_signal_connect (GTK_WINDOW(taskbar), "destroy", G_CALLBACK (on_launcher_destroy), NULL);
    
    // int layer shell for window
    gtk_layer_init_for_window (GTK_WINDOW(self));
    // anchor window to all edges
    gtk_layer_set_anchor (GTK_WINDOW(self), GTK_LAYER_SHELL_EDGE_LEFT, true);
    gtk_layer_set_anchor (GTK_WINDOW(self), GTK_LAYER_SHELL_EDGE_RIGHT, true);
    gtk_layer_set_anchor (GTK_WINDOW(self), GTK_LAYER_SHELL_EDGE_TOP, false);
    gtk_layer_set_anchor (GTK_WINDOW(self), GTK_LAYER_SHELL_EDGE_BOTTOM, true);
    gtk_layer_set_namespace(GTK_WINDOW(self), "diya-taskbar");
    // set margin on window
    for (int i = 0; i < GTK_LAYER_SHELL_EDGE_ENTRY_NUMBER; i++)
        gtk_layer_set_margin (GTK_WINDOW(self), i, 0);
    gtk_layer_set_layer (GTK_WINDOW(self), GTK_LAYER_SHELL_LAYER_TOP);
    gtk_layer_set_keyboard_mode (GTK_WINDOW(self), GTK_LAYER_SHELL_KEYBOARD_MODE_ON_DEMAND);
    // the top launcher shall be exclusive
    gtk_layer_auto_exclusive_zone_enable (GTK_WINDOW(self));

    g_object_bind_property(self->btn_toggle, "active", self, DIYA_PROP_TASKBAR_SHOW_DASHBOARD, G_BINDING_BIDIRECTIONAL);
}


static void diya_taskbar_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
{
    DiyaTaskbarWidget * self = DIYA_TASKBAR_WIDGET(object);

    switch (property_id)
    {
    case PROP_SHOW_DASHBOARD:
        self->active_dashboard = g_value_get_boolean(value);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}

static void diya_taskbar_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
{
    DiyaTaskbarWidget * self = DIYA_TASKBAR_WIDGET(object);
    switch (property_id)
    {
    case PROP_SHOW_DASHBOARD:
        g_value_set_boolean(value, self->active_dashboard);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}


/*
static void diya_dashboard_toggle(GtkToggleButton* btn, DiyaTaskbarWidget* self)
{
    g_warning("TODDLDLDL");
    if(gtk_toggle_button_get_active(btn))
    {
        gtk_layer_set_exclusive_zone(GTK_WINDOW(self), 0);
    }
    else
    {
        gtk_layer_auto_exclusive_zone_enable (GTK_WINDOW(self));
    }
}
    */


static void diya_taskbar_widget_class_init(DiyaTaskbarWidgetClass* class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(class);
    gobject_class->dispose = diya_taskbar_widget_dispose;
    gobject_class->set_property = diya_taskbar_set_property;
    gobject_class->get_property = diya_taskbar_get_property;

    gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (class),"/dev/iohub/diya/shell/taskbar.ui");
    gtk_widget_class_bind_template_child (GTK_WIDGET_CLASS (class), DiyaTaskbarWidget, btn_toggle);

    // gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (class), diya_dashboard_toggle);

    gtk_widget_class_set_css_name(GTK_WIDGET_CLASS(class), "diya-taskbar");
    //gtk_widget_class_bind_template_child(GTK_WIDGET_CLASS (class), ExampleAppWindow, stack);
    //gtk_widget_class_bind_template_callback (GTK_WIDGET_CLASS (class), search_text_changed);

    g_prop[PROP_SHOW_DASHBOARD] = g_param_spec_boolean(DIYA_PROP_TASKBAR_SHOW_DASHBOARD, NULL, "Show/hide dashboard", false, G_PARAM_READWRITE); // 
 
    g_object_class_install_properties (gobject_class, N_PROPERTIES, g_prop);
}